@@ -35,25 +35,25 @@ namespace TextDocumentUtils {
             const QTextBlock endBlock = document->findBlock( endPosition );
             const QRectF endBoundingRect = document->documentLayout()->blockBoundingRect( endBlock );
 
-            QTextLayout *startLayout = startBlock.layout();
-            QTextLayout *endLayout = endBlock.layout();
+            const QTextLayout *startLayout = startBlock.layout();
+            const QTextLayout *endLayout = endBlock.layout();
             if (!startLayout || !endLayout) {
                 qCWarning(OkularCoreDebug) << "Start or end layout not found" << startLayout << endLayout;
                 page = -1;
                 return;
             }
 
-            int startPos = startPosition - startBlock.position();
-            int endPos = endPosition - endBlock.position();
+            const int startPos = startPosition - startBlock.position();
+            const int endPos = endPosition - endBlock.position();
             const QTextLine startLine = startLayout->lineForTextPosition( startPos );
             const QTextLine endLine = endLayout->lineForTextPosition( endPos );
 
-            double x = startBoundingRect.x() + startLine.cursorToX( startPos );
-            double y = startBoundingRect.y() + startLine.y();
-            double r = endBoundingRect.x() + endLine.cursorToX( endPos );
-            double b = endBoundingRect.y() + endLine.y() + endLine.height();
+            const double x = startBoundingRect.x() + startLine.cursorToX( startPos );
+            const double y = startBoundingRect.y() + startLine.y();
+            const double r = endBoundingRect.x() + endLine.cursorToX( endPos );
+            const double b = endBoundingRect.y() + endLine.y() + endLine.height();
 
-            int offset = qRound( y ) % qRound( pageSize.height() );
+            const int offset = qRound( y ) % qRound( pageSize.height() );
 
             if ( x > r ) { // line break, so return a pseudo character on the start line
                 rect = QRectF( x / pageSize.width(), offset / pageSize.height(),
@@ -67,11 +67,71 @@ namespace TextDocumentUtils {
                            (r - x) / pageSize.width(), (b - y) / pageSize.height() );
         }
 
+        static QVector<QRectF> calculateBoundingRects( QTextDocument *document, int startPosition, int endPosition )
+        {
+            QVector<QRectF> result;
+
+            const QSizeF pageSize = document->pageSize();
+
+            const QTextBlock startBlock = document->findBlock( startPosition );
+            const QRectF startBoundingRect = document->documentLayout()->blockBoundingRect( startBlock );
+
+            const QTextBlock endBlock = document->findBlock( endPosition );
+            const QRectF endBoundingRect = document->documentLayout()->blockBoundingRect( endBlock );
+
+            const QTextLayout *startLayout = startBlock.layout();
+            const QTextLayout *endLayout = endBlock.layout();
+            if (!startLayout || !endLayout) {
+                qCWarning(OkularCoreDebug) << "Start or end layout not found" << startLayout << endLayout;
+                return {};
+            }
+
+            const int startPos = startPosition - startBlock.position();
+            const int endPos = endPosition - endBlock.position();
+            const QTextLine startLine = startLayout->lineForTextPosition( startPos );
+            const QTextLine endLine = endLayout->lineForTextPosition( endPos );
+
+            // This only works if both start and end layout are the same
+            if (startLayout == endLayout) {
+                Q_ASSERT( startBoundingRect == endBoundingRect );
+                for (int i = startLine.lineNumber(); i < endLine.lineNumber(); ++i) {
+                    const QTextLine line = startLayout->lineAt( i );
+                    // using startPos and endPos is fine, if the pos is out of bounds for that line, it'll return beginning and end of line respectively
+                    const double x = endBoundingRect.x() + line.cursorToX( startPos );
+                    const double y = endBoundingRect.y() + line.y();
+                    const double r = endBoundingRect.x() + line.cursorToX( endPos );
+                    const double b = endBoundingRect.y() + line.y() + endLine.height();
+
+                    result.append( QRectF( x / pageSize.width(), y / pageSize.height(),
+                                          (r - x) / pageSize.width(), (b - y) / pageSize.height() ) );
+                }
+
+                // The last line
+                const double x = endBoundingRect.x() + endLine.cursorToX( startPos );
+                const double y = endBoundingRect.y() + endLine.y();
+                const double r = endBoundingRect.x() + endLine.cursorToX( endPos );
+                const double b = endBoundingRect.y() + endLine.y() + endLine.height();
+
+                result.append( QRectF( x / pageSize.width(), y / pageSize.height(),
+                                      (r - x) / pageSize.width(), (b - y) / pageSize.height() ) );
+            } else {
+                const double x = startBoundingRect.x() + startLine.cursorToX( startPos );
+                const double y = startBoundingRect.y() + startLine.y();
+                const double r = endBoundingRect.x() + endLine.cursorToX( endPos );
+                const double b = endBoundingRect.y() + endLine.y() + endLine.height();
+
+                result.append( QRectF( x / pageSize.width(), y / pageSize.height(),
+                                  (r - x) / pageSize.width(), (b - y) / pageSize.height() ) );
+            }
+
+            return result;
+        }
+
         static void calculatePositions( QTextDocument *document, int page, int &start, int &end )
         {
             const QAbstractTextDocumentLayout *layout = document->documentLayout();
             const QSizeF pageSize = document->pageSize();
-            double margin = document->rootFrame()->frameFormat().margin();
+            const double margin = document->rootFrame()->frameFormat().margin();
 
             /**
              * Take the upper left and lower left corner including the margin
@@ -85,8 +145,8 @@ namespace TextDocumentUtils {
             const QSizeF pageSize = document->pageSize();
             const QRectF rect = document->documentLayout()->blockBoundingRect( block );
 
-            int page = qRound( rect.y() ) / qRound( pageSize.height() );
-            int offset = qRound( rect.y() ) % qRound( pageSize.height() );
+            const int page = qRound( rect.y() ) / qRound( pageSize.height() );
+            const int offset = qRound( rect.y() ) % qRound( pageSize.height() );
 
             Okular::DocumentViewport viewport( page );
             viewport.rePos.normalizedX = (double)rect.x() / (double)pageSize.width();
@@ -128,6 +188,21 @@ class TextDocumentGeneratorPrivate : public GeneratorPrivate
 
         void initializeGenerator();
 
+        struct LinkInfo
+        {
+          int page;
+          QRectF boundingRect;
+          Action *link;
+          bool ownsLink;
+        };
+
+        struct AnnotationInfo
+        {
+          int page;
+          QRectF boundingRect;
+          Annotation *annotation;
+        };
+
         Q_DECLARE_PUBLIC( TextDocumentGenerator )
 
         /* reimp */ QVariant metaData( const QString &key, const QVariant &option ) const override;
@@ -143,8 +218,8 @@ class TextDocumentGeneratorPrivate : public GeneratorPrivate
         void addMetaData( const QString &key, const QString &value, const QString &title );
         void addMetaData( DocumentInfo::Key, const QString &value );
 
-        void generateLinkInfos();
-        void generateAnnotationInfos();
+        QList<LinkInfo> generateLinkInfos() const;
+        QList<AnnotationInfo> generateAnnotationInfos() const;
         void generateTitleInfos();
 
         TextDocumentConverter *mConverter;
@@ -169,14 +244,6 @@ class TextDocumentGeneratorPrivate : public GeneratorPrivate
         };
         QList<LinkPosition> mLinkPositions;
 
-        struct LinkInfo
-        {
-          int page;
-          QRectF boundingRect;
-          Action *link;
-        };
-        QList<LinkInfo> mLinkInfos;
-
         struct AnnotationPosition
         {
           int startPosition;
@@ -185,14 +252,6 @@ class TextDocumentGeneratorPrivate : public GeneratorPrivate
         };
         QList<AnnotationPosition> mAnnotationPositions;
 
-        struct AnnotationInfo
-        {
-          int page;
-          QRectF boundingRect;
-          Annotation *annotation;
-        };
-        QList<AnnotationInfo> mAnnotationInfos;
-
         TextDocumentSettings *mGeneralSettings;
 
         QFont mFont;
