@@ -40,13 +40,10 @@ class FormFieldSignature;
 class Annotation;
 class MovieAction;
 class RenditionAction;
-class PixmapRequest;
 }
 
-class FormWidgetIface;
 class PageViewPrivate;
 
-class MagnifierView;
 
 class QGestureEvent;
 
@@ -97,7 +94,7 @@ Q_OBJECT
         void setCapability( ViewCapability capability, const QVariant &option ) override;
 
         QList< Okular::RegularAreaRect * > textSelections( const QPoint& start, const QPoint& end, int& firstpage );
-        Okular::RegularAreaRect * textSelectionForItem( PageViewItem * item, const QPoint & startPoint = QPoint(), const QPoint & endPoint = QPoint() );
+        Okular::RegularAreaRect * textSelectionForItem( const PageViewItem * item, const QPoint & startPoint = QPoint(), const QPoint & endPoint = QPoint() );
 
         void reparseConfig();
 
@@ -130,6 +127,8 @@ Q_OBJECT
         void slotToggleChangeColors();
         void slotSetChangeColors(bool active);
 
+        void slotSelectPage();
+
     Q_SIGNALS:
         void rightClick( const Okular::Page *, const QPoint & );
         void mouseBackButtonClick();
